@@ -1,15 +1,10 @@
-"""
-    sphinx.builders.manpage
-    ~~~~~~~~~~~~~~~~~~~~~~~
+"""Manual pages builder."""
 
-    Manual pages builder.
-
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+from __future__ import annotations
 
+import warnings
 from os import path
-from typing import Any, Dict, List, Set, Tuple, Union
+from typing import Any
 
 from docutils.frontend import OptionParser
 from docutils.io import FileOutput
@@ -18,10 +13,10 @@ from sphinx import addnodes
 from sphinx.application import Sphinx
 from sphinx.builders import Builder
 from sphinx.config import Config
-from sphinx.errors import NoUri
 from sphinx.locale import __
-from sphinx.util import logging, progress_message
+from sphinx.util import logging
 from sphinx.util.console import darkgreen  # type: ignore
+from sphinx.util.display import progress_message
 from sphinx.util.nodes import inline_all_toctrees
 from sphinx.util.osutil import ensuredir, make_filename_from_project
 from sphinx.writers.manpage import ManualPageTranslator, ManualPageWriter
@@ -38,28 +33,30 @@ class ManualPageBuilder(Builder):
     epilog = __('The manual pages are in %(outdir)s.')
 
     default_translator_class = ManualPageTranslator
-    supported_image_types: List[str] = []
+    supported_image_types: list[str] = []
 
     def init(self) -> None:
         if not self.config.man_pages:
             logger.warning(__('no "man_pages" config value found; no manual pages '
                               'will be written'))
 
-    def get_outdated_docs(self) -> Union[str, List[str]]:
+    def get_outdated_docs(self) -> str | list[str]:
         return 'all manpages'  # for now
 
-    def get_target_uri(self, docname: str, typ: str = None) -> str:
-        if typ == 'token':
-            return ''
-        raise NoUri(docname, typ)
+    def get_target_uri(self, docname: str, typ: str | None = None) -> str:
+        return ''
 
     @progress_message(__('writing'))
     def write(self, *ignored: Any) -> None:
         docwriter = ManualPageWriter(self)
-        docsettings: Any = OptionParser(
-            defaults=self.env.settings,
-            components=(docwriter,),
-            read_config_files=True).get_default_values()
+        with warnings.catch_warnings():
+            warnings.filterwarnings('ignore', category=DeprecationWarning)
+            # DeprecationWarning: The frontend.OptionParser class will be replaced
+            # by a subclass of argparse.ArgumentParser in Docutils 0.21 or later.
+            docsettings: Any = OptionParser(
+                defaults=self.env.settings,
+                components=(docwriter,),
+                read_config_files=True).get_default_values()
 
         for info in self.config.man_pages:
             docname, name, description, authors, section = info
@@ -79,10 +76,11 @@ class ManualPageBuilder(Builder):
             docsettings.section = section
 
             if self.config.man_make_section_directory:
-                ensuredir(path.join(self.outdir, str(section)))
-                targetname = '%s/%s.%s' % (section, name, section)
+                dirname = 'man%s' % section
+                ensuredir(path.join(self.outdir, dirname))
+                targetname = f'{dirname}/{name}.{section}'
             else:
-                targetname = '%s.%s' % (name, section)
+                targetname = f'{name}.{section}'
 
             logger.info(darkgreen(targetname) + ' { ', nonl=True)
             destination = FileOutput(
@@ -90,14 +88,14 @@ class ManualPageBuilder(Builder):
                 encoding='utf-8')
 
             tree = self.env.get_doctree(docname)
-            docnames: Set[str] = set()
+            docnames: set[str] = set()
             largetree = inline_all_toctrees(self, docnames, docname, tree,
                                             darkgreen, [docname])
             largetree.settings = docsettings
             logger.info('} ', nonl=True)
             self.env.resolve_references(largetree, docname, self)
             # remove pending_xref nodes
-            for pendingnode in largetree.traverse(addnodes.pending_xref):
+            for pendingnode in largetree.findall(addnodes.pending_xref):
                 pendingnode.replace_self(pendingnode.children)
 
             docwriter.write(largetree, destination)
@@ -106,19 +104,19 @@ class ManualPageBuilder(Builder):
         pass
 
 
-def default_man_pages(config: Config) -> List[Tuple[str, str, str, List[str], int]]:
+def default_man_pages(config: Config) -> list[tuple[str, str, str, list[str], int]]:
     """ Better default man_pages settings. """
     filename = make_filename_from_project(config.project)
-    return [(config.root_doc, filename, '%s %s' % (config.project, config.release),
+    return [(config.root_doc, filename, f'{config.project} {config.release}',
              [config.author], 1)]
 
 
-def setup(app: Sphinx) -> Dict[str, Any]:
+def setup(app: Sphinx) -> dict[str, Any]:
     app.add_builder(ManualPageBuilder)
 
-    app.add_config_value('man_pages', default_man_pages, None)
-    app.add_config_value('man_show_urls', False, None)
-    app.add_config_value('man_make_section_directory', True, None)
+    app.add_config_value('man_pages', default_man_pages, False)
+    app.add_config_value('man_show_urls', False, False)
+    app.add_config_value('man_make_section_directory', False, False)
 
     return {
         'version': 'builtin',
