@@ -1,25 +1,26 @@
+"""Utility code for "Doc fields".
+
+"Doc fields" are reST field lists in object descriptions that will
+be domain-specifically transformed to a more appealing presentation.
 """
-    sphinx.util.docfields
-    ~~~~~~~~~~~~~~~~~~~~~
+from __future__ import annotations
 
-    "Doc fields" are reST field lists in object descriptions that will
-    be domain-specifically transformed to a more appealing presentation.
-
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
-
-from typing import TYPE_CHECKING, Any, Dict, List, Tuple, Type, Union, cast
+from typing import TYPE_CHECKING, Any, List, Tuple, cast
 
 from docutils import nodes
 from docutils.nodes import Node
+from docutils.parsers.rst.states import Inliner
 
 from sphinx import addnodes
 from sphinx.environment import BuildEnvironment
+from sphinx.locale import __
+from sphinx.util import logging
 from sphinx.util.typing import TextlikeNode
 
 if TYPE_CHECKING:
-    from sphinx.directive import ObjectDescription
+    from sphinx.directives import ObjectDescription
+
+logger = logging.getLogger(__name__)
 
 
 def _is_single_paragraph(node: nodes.field_body) -> bool:
@@ -51,7 +52,7 @@ class Field:
     is_grouped = False
     is_typed = False
 
-    def __init__(self, name: str, names: Tuple[str, ...] = (), label: str = None,
+    def __init__(self, name: str, names: tuple[str, ...] = (), label: str = None,
                  has_arg: bool = True, rolename: str = None, bodyrolename: str = None) -> None:
         self.name = name
         self.names = names
@@ -61,40 +62,59 @@ class Field:
         self.bodyrolename = bodyrolename
 
     def make_xref(self, rolename: str, domain: str, target: str,
-                  innernode: Type[TextlikeNode] = addnodes.literal_emphasis,
-                  contnode: Node = None, env: BuildEnvironment = None) -> Node:
+                  innernode: type[TextlikeNode] = addnodes.literal_emphasis,
+                  contnode: Node = None, env: BuildEnvironment = None,
+                  inliner: Inliner = None, location: Node = None) -> Node:
+        # note: for backwards compatibility env is last, but not optional
+        assert env is not None
+        assert (inliner is None) == (location is None), (inliner, location)
         if not rolename:
             return contnode or innernode(target, target)
-        refnode = addnodes.pending_xref('', refdomain=domain, refexplicit=False,
-                                        reftype=rolename, reftarget=target)
-        refnode += contnode or innernode(target, target)
-        if env:
+        # The domain is passed from DocFieldTransformer. So it surely exists.
+        # So we don't need to take care the env.get_domain() raises an exception.
+        role = env.get_domain(domain).role(rolename)
+        if role is None or inliner is None:
+            if role is None and inliner is not None:
+                msg = __("Problem in %s domain: field is supposed "
+                         "to use role '%s', but that role is not in the domain.")
+                logger.warning(__(msg), domain, rolename, location=location)
+            refnode = addnodes.pending_xref('', refdomain=domain, refexplicit=False,
+                                            reftype=rolename, reftarget=target)
+            refnode += contnode or innernode(target, target)
             env.get_domain(domain).process_field_xref(refnode)
-        return refnode
+            return refnode
+        lineno = logging.get_source_line(location)[1]
+        ns, messages = role(rolename, target, target, lineno, inliner, {}, [])
+        return nodes.inline(target, '', *ns)
 
     def make_xrefs(self, rolename: str, domain: str, target: str,
-                   innernode: Type[TextlikeNode] = addnodes.literal_emphasis,
-                   contnode: Node = None, env: BuildEnvironment = None) -> List[Node]:
-        return [self.make_xref(rolename, domain, target, innernode, contnode, env)]
+                   innernode: type[TextlikeNode] = addnodes.literal_emphasis,
+                   contnode: Node = None, env: BuildEnvironment = None,
+                   inliner: Inliner = None, location: Node = None) -> list[Node]:
+        return [self.make_xref(rolename, domain, target, innernode, contnode,
+                               env, inliner, location)]
 
-    def make_entry(self, fieldarg: str, content: List[Node]) -> Tuple[str, List[Node]]:
+    def make_entry(self, fieldarg: str, content: list[Node]) -> tuple[str, list[Node]]:
         return (fieldarg, content)
 
-    def make_field(self, types: Dict[str, List[Node]], domain: str,
-                   item: Tuple, env: BuildEnvironment = None) -> nodes.field:
+    def make_field(self, types: dict[str, list[Node]], domain: str,
+                   item: tuple, env: BuildEnvironment = None,
+                   inliner: Inliner = None, location: Node = None) -> nodes.field:
         fieldarg, content = item
         fieldname = nodes.field_name('', self.label)
         if fieldarg:
             fieldname += nodes.Text(' ')
             fieldname.extend(self.make_xrefs(self.rolename, domain,
-                                             fieldarg, nodes.Text, env=env))
+                                             fieldarg, nodes.Text,
+                                             env=env, inliner=inliner, location=location))
 
         if len(content) == 1 and (
                 isinstance(content[0], nodes.Text) or
                 (isinstance(content[0], nodes.inline) and len(content[0]) == 1 and
                  isinstance(content[0][0], nodes.Text))):
             content = self.make_xrefs(self.bodyrolename, domain,
-                                      content[0].astext(), contnode=content[0], env=env)
+                                      content[0].astext(), contnode=content[0],
+                                      env=env, inliner=inliner, location=location)
         fieldbody = nodes.field_body('', nodes.paragraph('', '', *content))
         return nodes.field('', fieldname, fieldbody)
 
@@ -115,19 +135,21 @@ class GroupedField(Field):
     is_grouped = True
     list_type = nodes.bullet_list
 
-    def __init__(self, name: str, names: Tuple[str, ...] = (), label: str = None,
+    def __init__(self, name: str, names: tuple[str, ...] = (), label: str = None,
                  rolename: str = None, can_collapse: bool = False) -> None:
         super().__init__(name, names, label, True, rolename)
         self.can_collapse = can_collapse
 
-    def make_field(self, types: Dict[str, List[Node]], domain: str,
-                   items: Tuple, env: BuildEnvironment = None) -> nodes.field:
+    def make_field(self, types: dict[str, list[Node]], domain: str,
+                   items: tuple, env: BuildEnvironment = None,
+                   inliner: Inliner = None, location: Node = None) -> nodes.field:
         fieldname = nodes.field_name('', self.label)
         listnode = self.list_type()
         for fieldarg, content in items:
             par = nodes.paragraph()
             par.extend(self.make_xrefs(self.rolename, domain, fieldarg,
-                                       addnodes.literal_strong, env=env))
+                                       addnodes.literal_strong,
+                                       env=env, inliner=inliner, location=location))
             par += nodes.Text(' -- ')
             par += content
             listnode += nodes.list_item('', par)
@@ -162,15 +184,16 @@ class TypedField(GroupedField):
     """
     is_typed = True
 
-    def __init__(self, name: str, names: Tuple[str, ...] = (), typenames: Tuple[str, ...] = (),
+    def __init__(self, name: str, names: tuple[str, ...] = (), typenames: tuple[str, ...] = (),
                  label: str = None, rolename: str = None, typerolename: str = None,
                  can_collapse: bool = False) -> None:
         super().__init__(name, names, label, rolename, can_collapse)
         self.typenames = typenames
         self.typerolename = typerolename
 
-    def make_field(self, types: Dict[str, List[Node]], domain: str,
-                   items: Tuple, env: BuildEnvironment = None) -> nodes.field:
+    def make_field(self, types: dict[str, list[Node]], domain: str,
+                   items: tuple, env: BuildEnvironment = None,
+                   inliner: Inliner = None, location: Node = None) -> nodes.field:
         def handle_item(fieldarg: str, content: str) -> nodes.paragraph:
             par = nodes.paragraph()
             par.extend(self.make_xrefs(self.rolename, domain, fieldarg,
@@ -184,7 +207,8 @@ class TypedField(GroupedField):
                 if len(fieldtype) == 1 and isinstance(fieldtype[0], nodes.Text):
                     typename = fieldtype[0].astext()
                     par.extend(self.make_xrefs(self.typerolename, domain, typename,
-                                               addnodes.literal_emphasis, env=env))
+                                               addnodes.literal_emphasis, env=env,
+                                               inliner=inliner, location=location))
                 else:
                     par += fieldtype
                 par += nodes.Text(')')
@@ -209,9 +233,9 @@ class DocFieldTransformer:
     Transforms field lists in "doc field" syntax into better-looking
     equivalents, using the field type definitions given on a domain.
     """
-    typemap: Dict[str, Tuple[Field, bool]] = None
+    typemap: dict[str, tuple[Field, bool]]
 
-    def __init__(self, directive: "ObjectDescription") -> None:
+    def __init__(self, directive: ObjectDescription) -> None:
         self.directive = directive
 
         self.typemap = directive.get_field_type_map()
@@ -227,9 +251,9 @@ class DocFieldTransformer:
         """Transform a single field list *node*."""
         typemap = self.typemap
 
-        entries: List[Union[nodes.field, Tuple[Field, Any]]] = []
-        groupindices: Dict[str, int] = {}
-        types: Dict[str, Dict] = {}
+        entries: list[nodes.field | tuple[Field, Any, Node]] = []
+        groupindices: dict[str, int] = {}
+        types: dict[str, dict] = {}
 
         # step 1: traverse all fields and collect field types and content
         for field in cast(List[nodes.field], node):
@@ -271,7 +295,7 @@ class DocFieldTransformer:
                         self.directive.domain,
                         target,
                         contnode=content[0],
-                        env=self.directive.state.document.settings.env
+                        env=self.directive.state.document.settings.env,
                     )
                     if _is_single_paragraph(field_body):
                         paragraph = cast(nodes.paragraph, field_body[0])
@@ -289,8 +313,7 @@ class DocFieldTransformer:
             if is_typefield:
                 # filter out only inline nodes; others will result in invalid
                 # markup being written out
-                content = [n for n in content if isinstance(n, nodes.Inline) or
-                           isinstance(n, nodes.Text)]
+                content = [n for n in content if isinstance(n, (nodes.Inline, nodes.Text))]
                 if content:
                     types.setdefault(typename, {})[fieldarg] = content
                 continue
@@ -298,7 +321,7 @@ class DocFieldTransformer:
             # also support syntax like ``:param type name:``
             if typedesc.is_typed:
                 try:
-                    argtype, argname = fieldarg.split(None, 1)
+                    argtype, argname = fieldarg.rsplit(None, 1)
                 except ValueError:
                     pass
                 else:
@@ -317,16 +340,16 @@ class DocFieldTransformer:
             # get one entry per field
             if typedesc.is_grouped:
                 if typename in groupindices:
-                    group = cast(Tuple[Field, List], entries[groupindices[typename]])
+                    group = cast(Tuple[Field, List, Node], entries[groupindices[typename]])
                 else:
                     groupindices[typename] = len(entries)
-                    group = (typedesc, [])
+                    group = (typedesc, [], field)
                     entries.append(group)
                 new_entry = typedesc.make_entry(fieldarg, [translatable_content])
                 group[1].append(new_entry)
             else:
                 new_entry = typedesc.make_entry(fieldarg, [translatable_content])
-                entries.append((typedesc, new_entry))
+                entries.append((typedesc, new_entry, field))
 
         # step 2: all entries are collected, construct the new field list
         new_list = nodes.field_list()
@@ -335,10 +358,11 @@ class DocFieldTransformer:
                 # pass-through old field
                 new_list += entry
             else:
-                fieldtype, items = entry
+                fieldtype, items, location = entry
                 fieldtypes = types.get(fieldtype.name, {})
                 env = self.directive.state.document.settings.env
-                new_list += fieldtype.make_field(fieldtypes, self.directive.domain,
-                                                 items, env=env)
+                inliner = self.directive.state.inliner
+                new_list += fieldtype.make_field(fieldtypes, self.directive.domain, items,
+                                                 env=env, inliner=inliner, location=location)
 
         node.replace_self(new_list)
