@@ -4,7 +4,6 @@ from __future__ import annotations
 
 import os
 import re
-import warnings
 from contextlib import contextmanager
 from copy import copy
 from os import path
@@ -371,14 +370,6 @@ class NullReporter(Reporter):
         super().__init__('', 999, 4)
 
 
-def is_html5_writer_available() -> bool:
-    from sphinx.deprecation import RemovedInSphinx70Warning
-
-    warnings.warn('is_html5_writer_available() is deprecated.',
-                  RemovedInSphinx70Warning)
-    return True
-
-
 @contextmanager
 def switch_source_input(state: State, content: StringList) -> Generator[None, None, None]:
     """Switch current source input of state temporarily."""
