@@ -4,7 +4,6 @@ from __future__ import annotations
 
 import os
 import re
-import warnings
 from datetime import datetime, timezone
 from os import path
 from typing import TYPE_CHECKING, Callable, Generator, NamedTuple
@@ -13,7 +12,6 @@ import babel.dates
 from babel.messages.mofile import write_mo
 from babel.messages.pofile import read_po
 
-from sphinx.deprecation import RemovedInSphinx70Warning
 from sphinx.errors import SphinxError
 from sphinx.locale import __
 from sphinx.util import logging
@@ -171,11 +169,6 @@ date_format_re = re.compile('(%s)' % '|'.join(date_format_mappings))
 
 def babel_format_date(date: datetime, format: str, locale: str,
                       formatter: Callable = babel.dates.format_date) -> str:
-    if locale is None:
-        warnings.warn('The locale argument for babel_format_date() becomes required.',
-                      RemovedInSphinx70Warning)
-        locale = 'en'
-
     # Check if we have the tzinfo attribute. If not we cannot do any time
     # related formats.
     if not hasattr(date, 'tzinfo'):
@@ -193,7 +186,7 @@ def babel_format_date(date: datetime, format: str, locale: str,
 
 
 def format_date(
-    format: str, date: datetime | None = None, language: str | None = None,
+    format: str, *, date: datetime | None = None, language: str,
 ) -> str:
     if date is None:
         # If time is not specified, try to use $SOURCE_DATE_EPOCH variable
@@ -204,11 +197,6 @@ def format_date(
         else:
             date = datetime.now(timezone.utc).astimezone()
 
-    if language is None:
-        warnings.warn('The language argument for format_date() becomes required.',
-                      RemovedInSphinx70Warning)
-        language = 'en'
-
     result = []
     tokens = date_format_re.split(format)
     for token in tokens:
